@@ -31,6 +31,15 @@
 #include <set>
 #include <utility>
 
+// Managing compiler warnings for SDL headers
+#if defined( __GNUC__ )
+#pragma GCC diagnostic push
+
+#pragma GCC diagnostic ignored "-Wdouble-promotion"
+#pragma GCC diagnostic ignored "-Wold-style-cast"
+#pragma GCC diagnostic ignored "-Wswitch-default"
+#endif
+
 #include <SDL_error.h>
 #include <SDL_events.h>
 #include <SDL_gamecontroller.h>
@@ -45,12 +54,17 @@
 #include <SDL_version.h>
 #include <SDL_video.h>
 
+// Managing compiler warnings for SDL headers
+#if defined( __GNUC__ )
+#pragma GCC diagnostic pop
+#endif
+
 #include "audio.h"
 #include "image.h"
 #include "logging.h"
+#include "math_tools.h"
 #include "render_processor.h"
 #include "screen.h"
-#include "tools.h"
 
 namespace
 {
@@ -126,6 +140,26 @@ namespace
             if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
                 return '9';
             break;
+        case fheroes2::Key::KEY_KP_PERIOD:
+            if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
+                return '.';
+            break;
+        case fheroes2::Key::KEY_KP_DIVIDE:
+            if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
+                return '/';
+            break;
+        case fheroes2::Key::KEY_KP_MULTIPLY:
+            if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
+                return '*';
+            break;
+        case fheroes2::Key::KEY_KP_MINUS:
+            if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
+                return '-';
+            break;
+        case fheroes2::Key::KEY_KP_PLUS:
+            if ( fheroes2::KeyModifier::KEY_MODIFIER_NUM & mod )
+                return '+';
+            break;
         case fheroes2::Key::KEY_MINUS:
             return ( fheroes2::KeyModifier::KEY_MODIFIER_SHIFT & mod ? '_' : '-' );
         case fheroes2::Key::KEY_EQUALS:
@@ -331,16 +365,22 @@ namespace EventProcessing
 
         static void initTouchpad()
         {
-            const int touchNumber = SDL_GetNumTouchDevices();
-            if ( touchNumber > 0 ) {
-                fheroes2::cursor().enableSoftwareEmulation( true );
 #if SDL_VERSION_ATLEAST( 2, 0, 10 )
-                const SDL_bool value = SDL_SetHint( SDL_HINT_TOUCH_MOUSE_EVENTS, "0" );
-                if ( value != SDL_TRUE ) {
-                    ERROR_LOG( "Failed to set SDL_HINT_TOUCH_MOUSE_EVENTS." )
-                }
-#endif
+            if ( SDL_SetHint( SDL_HINT_MOUSE_TOUCH_EVENTS, "0" ) != SDL_TRUE ) {
+                ERROR_LOG( "Failed to set the " SDL_HINT_MOUSE_TOUCH_EVENTS " hint." )
             }
+#endif
+#if SDL_VERSION_ATLEAST( 2, 0, 6 )
+            if ( SDL_SetHint( SDL_HINT_TOUCH_MOUSE_EVENTS, "0" ) != SDL_TRUE ) {
+                ERROR_LOG( "Failed to set the " SDL_HINT_TOUCH_MOUSE_EVENTS " hint." )
+            }
+#endif
+
+            if ( SDL_GetNumTouchDevices() <= 0 ) {
+                return;
+            }
+
+            fheroes2::cursor().enableSoftwareEmulation( true );
         }
 
         void initController()
@@ -389,19 +429,27 @@ namespace EventProcessing
             SDL_Event event;
 
             while ( SDL_PollEvent( &event ) ) {
+                // Most SDL events should be processed sequentially one event at a time, but for some
+                // event types, the processing of intermediate events may be skipped in order to gain
+                // overall event processing speed.
+                bool processImmediately = true;
+
                 switch ( event.type ) {
                 case SDL_WINDOWEVENT:
                     if ( event.window.event == SDL_WINDOWEVENT_CLOSE ) {
-                        // A special case since we need to exit the loop.
                         if ( allowExit ) {
                             // Try to perform clear exit to catch all memory leaks, for example.
                             return false;
                         }
+                        processImmediately = false;
                         break;
                     }
                     if ( onWindowEvent( event.window ) ) {
                         updateDisplay = true;
                     }
+                    else {
+                        processImmediately = false;
+                    }
                     break;
                 case SDL_KEYDOWN:
                 case SDL_KEYUP:
@@ -409,6 +457,7 @@ namespace EventProcessing
                     break;
                 case SDL_MOUSEMOTION:
                     onMouseMotionEvent( eventHandler, event.motion );
+                    processImmediately = false;
                     break;
                 case SDL_MOUSEBUTTONDOWN:
                 case SDL_MOUSEBUTTONUP:
@@ -434,9 +483,11 @@ namespace EventProcessing
                     // SDL requires joystick events to be enabled in order to handle controller events.
                     // This is because the controller related code depends on the joystick related code.
                     // See SDL_gamecontroller.c within SDL source code for implementation details.
+                    processImmediately = false;
                     break;
                 case SDL_CONTROLLERAXISMOTION:
                     onControllerAxisEvent( eventHandler, event.caxis );
+                    processImmediately = false;
                     break;
                 case SDL_CONTROLLERBUTTONDOWN:
                 case SDL_CONTROLLERBUTTONUP:
@@ -446,6 +497,9 @@ namespace EventProcessing
                 case SDL_FINGERUP:
                 case SDL_FINGERMOTION:
                     onTouchEvent( eventHandler, event.tfinger );
+                    if ( event.type == SDL_FINGERMOTION ) {
+                        processImmediately = false;
+                    }
                     break;
                 case SDL_RENDER_TARGETS_RESET:
                     // We need to just update the screen. This event usually happens when we switch between fullscreen and windowed modes.
@@ -466,6 +520,7 @@ namespace EventProcessing
                         // Try to perform clear exit to catch all memory leaks, for example.
                         return false;
                     }
+                    processImmediately = false;
                     break;
                 case SDL_APP_LOWMEMORY:
                     // According to SDL this event can only happen on Android or iOS.
@@ -478,8 +533,16 @@ namespace EventProcessing
                     assert( eventTypeStatus.count( event.type ) == 0 );
 
                     // This is a new event type which we do not handle. It might have been added in a newer version of SDL.
+                    processImmediately = false;
                     break;
                 }
+
+                // If the current event does require immediate processing, then we need to return immediately.
+                if ( processImmediately ) {
+                    break;
+                }
+
+                // Otherwise, we can process it later along with the newly received events, if any.
             }
 
             return true;
@@ -495,8 +558,7 @@ namespace EventProcessing
 
         static void setEventProcessingState( const uint32_t eventType, const bool enable )
         {
-            const auto [dummy, inserted] = eventTypeStatus.emplace( eventType );
-            if ( !inserted ) {
+            if ( const auto [dummy, inserted] = eventTypeStatus.emplace( eventType ); !inserted ) {
                 assert( 0 );
             }
 
@@ -1057,9 +1119,8 @@ namespace fheroes2
             }
             break;
         case fheroes2::Key::KEY_DELETE:
-            if ( !res.empty() ) {
-                if ( pos < res.size() )
-                    res.erase( pos, 1 );
+            if ( pos < res.size() ) {
+                res.erase( pos, 1 );
             }
             break;
 
