@@ -31,7 +31,6 @@
 #include <utility>
 #include <vector>
 
-#include "battle.h"
 #include "battle_animation.h"
 #include "battle_board.h"
 #include "battle_troop.h"
@@ -40,6 +39,7 @@
 #include "icn.h"
 #include "image.h"
 #include "math_base.h"
+#include "screen.h"
 #include "spell.h"
 #include "ui_button.h"
 #include "ui_text.h"
@@ -64,6 +64,11 @@ namespace Battle
     class Tower;
     class Units;
 
+    struct TargetInfo;
+    struct TargetsInfo;
+
+    enum class CastleDefenseStructure : int;
+
     void DialogBattleSettings();
     bool DialogBattleSurrender( const HeroBase & hero, uint32_t cost, Kingdom & kingdom );
 
@@ -112,14 +117,14 @@ namespace Battle
     class OpponentSprite
     {
     public:
-        OpponentSprite( const fheroes2::Rect & area, const HeroBase * hero, const bool isReflect );
+        OpponentSprite( const fheroes2::Rect & area, HeroBase * hero, const bool isReflect );
         OpponentSprite( const OpponentSprite & ) = delete;
 
         OpponentSprite & operator=( const OpponentSprite & ) = delete;
 
         const fheroes2::Rect & GetArea() const
         {
-            return pos;
+            return _area;
         }
 
         fheroes2::Point GetCastPosition() const;
@@ -129,16 +134,16 @@ namespace Battle
         bool updateAnimationState();
 
         void SetAnimation( const int rule );
-        void IncreaseAnimFrame( const bool loop = false );
+        void IncreaseAnimFrame();
 
         bool isFinishFrame() const
         {
             return _currentAnim.isLastFrame();
         }
 
-        const HeroBase * GetHero() const
+        HeroBase * GetHero() const
         {
-            return base;
+            return _heroBase;
         }
 
         fheroes2::Point Offset() const
@@ -157,36 +162,41 @@ namespace Battle
         };
 
     private:
-        const HeroBase * base;
+        HeroBase * _heroBase;
         AnimationSequence _currentAnim;
-        int _animationType;
-        RandomizedDelay _idleTimer;
+        int _animationType{ OP_STATIC };
+        RandomizedDelay _idleTimer{ 8000 };
 
-        int _heroIcnId;
-        bool reflect;
-        fheroes2::Rect pos;
+        int _heroIcnId{ ICN::UNKNOWN };
+        bool _isFlippedHorizontally;
+        fheroes2::Rect _area;
         fheroes2::Point _offset;
     };
 
-    class Status : public fheroes2::Rect
+    class Status final : public fheroes2::Rect
     {
     public:
         Status();
         Status( const Status & ) = delete;
+        ~Status() = default;
 
         Status & operator=( const Status & ) = delete;
 
-        void SetPosition( const int32_t cx, const int32_t cy );
-
-        void SetLogs( StatusListBox * logs )
+        void setPosition( const int32_t cx, const int32_t cy )
         {
-            listlog = logs;
+            x = cx;
+            y = cy;
         }
 
-        void SetMessage( const std::string & messageString, const bool top = false );
-        void Redraw( fheroes2::Image & output ) const;
+        void setLogs( StatusListBox * logs )
+        {
+            _battleStatusLog = logs;
+        }
 
-        const std::string & GetMessage() const
+        void setMessage( std::string messageString, const bool top );
+        void redraw( fheroes2::Image & output ) const;
+
+        const std::string & getMessage() const
         {
             return _lastMessage;
         }
@@ -196,23 +206,30 @@ namespace Battle
     private:
         fheroes2::Text _upperText;
         fheroes2::Text _lowerText;
-        const fheroes2::Sprite & back1;
-        const fheroes2::Sprite & back2;
+        const fheroes2::Sprite & _upperBackground;
+        const fheroes2::Sprite & _lowerBackground;
         std::string _lastMessage;
-        StatusListBox * listlog;
+        StatusListBox * _battleStatusLog{ nullptr };
     };
 
-    class TurnOrder : public fheroes2::Rect
+    class TurnOrder final : public fheroes2::Rect
     {
     public:
-        TurnOrder();
+        TurnOrder() = default;
         TurnOrder( const TurnOrder & ) = delete;
+        ~TurnOrder() = default;
 
         TurnOrder & operator=( const TurnOrder & ) = delete;
 
-        void Set( const fheroes2::Rect & rt, const std::shared_ptr<const Units> & units, const int army2Color );
-        void Redraw( const Unit * current, const uint8_t currentUnitColor, fheroes2::Image & output );
-        void QueueEventProcessing( std::string & msg, const fheroes2::Point & offset ) const;
+        void set( const fheroes2::Rect & roi, const std::shared_ptr<const Units> & units, const int opponentColor )
+        {
+            _area = roi;
+            _orderOfUnits = units;
+            _opponentColor = opponentColor;
+        }
+
+        void redraw( const Unit * current, const uint8_t currentUnitColor, fheroes2::Image & output );
+        void queueEventProcessing( std::string & msg, const fheroes2::Point & offset ) const;
 
     private:
         enum ArmyColor : uint8_t
@@ -229,11 +246,11 @@ namespace Battle
 
         using UnitPos = std::pair<const Unit *, fheroes2::Rect>;
 
-        void RedrawUnit( const fheroes2::Rect & pos, const Battle::Unit & unit, const bool revert, const bool isCurrentUnit, const uint8_t currentUnitColor,
-                         fheroes2::Image & output ) const;
+        void _redrawUnit( const fheroes2::Rect & pos, const Battle::Unit & unit, const bool revert, const bool isCurrentUnit, const uint8_t currentUnitColor,
+                          fheroes2::Image & output ) const;
 
-        std::weak_ptr<const Units> _orders;
-        int _army2Color;
+        std::weak_ptr<const Units> _orderOfUnits;
+        int _opponentColor{ 0 };
         fheroes2::Rect _area;
         std::vector<UnitPos> _rects;
     };
@@ -241,26 +258,37 @@ namespace Battle
     class PopupDamageInfo : public Dialog::FrameBorder
     {
     public:
-        PopupDamageInfo();
+        PopupDamageInfo()
+            : Dialog::FrameBorder( 5 )
+        {
+            // Do nothing.
+        }
+
         PopupDamageInfo( const PopupDamageInfo & ) = delete;
 
         PopupDamageInfo & operator=( const PopupDamageInfo & ) = delete;
 
-        void setBattleUIRect( const fheroes2::Rect & battleUIRect );
-        void SetAttackInfo( const Cell * cell, const Unit * attacker, const Unit * defender );
-        void SetSpellAttackInfo( const Cell * cell, const HeroBase * hero, const Unit * defender, const Spell spell );
-        void Reset();
-        void Redraw() const;
+        void setBattleUIRect( const fheroes2::Rect & battleUIRect )
+        {
+            _battleUIRect = battleUIRect;
+        }
+
+        void setAttackInfo( const Unit * attacker, const Unit * defender );
+        void setSpellAttackInfo( const HeroBase * hero, const Unit * defender, const Spell & spell );
+        void reset();
+        void redraw() const;
 
     private:
-        bool SetDamageInfoBase( const Cell * cell, const Unit * defender );
+        bool _setDamageInfoBase( const Unit * defender );
+        void _makeDamageImage();
 
+        fheroes2::Sprite _damageImage;
         fheroes2::Rect _battleUIRect;
-        const Cell * _cell;
-        const Unit * _defender;
-        uint32_t _minDamage;
-        uint32_t _maxDamage;
-        bool _redraw;
+        const Battle::Unit * _defender{ nullptr };
+        uint32_t _minDamage{ 0 };
+        uint32_t _maxDamage{ 0 };
+        bool _redraw{ false };
+        bool _needDelay{ true };
     };
 
     class Interface
@@ -294,7 +322,7 @@ namespace Battle
 
         fheroes2::Point getRelativeMouseCursorPos() const;
 
-        void SetStatus( const std::string & message, const bool top = false );
+        void setStatus( const std::string & message, const bool top );
         void SetOrderOfUnits( const std::shared_ptr<const Units> & units );
         void FadeArena( const bool clearMessageLog );
 
@@ -312,10 +340,11 @@ namespace Battle
         void RedrawActionLuck( const Unit & unit );
         void RedrawActionTowerPart1( const Tower & tower, const Unit & defender );
         void RedrawActionTowerPart2( const Tower & tower, const TargetInfo & target );
-        void RedrawActionCatapultPart1( const CastleDefenseElement catapultTarget, const bool isHit );
-        void RedrawActionCatapultPart2( const CastleDefenseElement catapultTarget );
+        void RedrawActionCatapultPart1( const CastleDefenseStructure catapultTarget, const bool isHit );
+        void RedrawActionCatapultPart2( const CastleDefenseStructure catapultTarget );
         void RedrawActionTeleportSpell( Unit & target, const int32_t dst );
-        void RedrawActionEarthQuakeSpell( const std::vector<CastleDefenseElement> & targets );
+        void redrawActionEarthquakeSpellPart1( const std::vector<CastleDefenseStructure> & targets );
+        void redrawActionEarthquakeSpellPart2( const std::vector<CastleDefenseStructure> & targets );
         void RedrawActionSummonElementalSpell( Unit & target );
         void RedrawActionMirrorImageSpell( const Unit & target, const Position & pos );
         void RedrawActionSkipStatus( const Unit & unit );
@@ -408,7 +437,7 @@ namespace Battle
         Dialog::FrameBorder border;
 
         fheroes2::Rect _interfacePosition;
-        fheroes2::Rect _surfaceInnerArea;
+        fheroes2::Rect _surfaceInnerArea{ 0, 0, fheroes2::Display::DEFAULT_WIDTH, fheroes2::Display::DEFAULT_HEIGHT };
         fheroes2::Image _mainSurface;
         fheroes2::Image _battleGround;
         fheroes2::Image _hexagonGrid;
@@ -416,8 +445,8 @@ namespace Battle
         fheroes2::Image _hexagonGridShadow;
         fheroes2::Image _hexagonCursorShadow;
 
-        int icn_cbkg;
-        int icn_frng;
+        int _battleGroundIcn{ ICN::UNKNOWN };
+        int _borderObjectsIcn{ ICN::UNKNOWN };
 
         fheroes2::Button btn_auto;
         fheroes2::Button btn_settings;
@@ -427,32 +456,34 @@ namespace Battle
         std::unique_ptr<OpponentSprite> _opponent1;
         std::unique_ptr<OpponentSprite> _opponent2;
 
-        Spell humanturn_spell;
-        bool humanturn_exit;
-        bool humanturn_redraw;
-        uint32_t animation_flags_frame;
-        int catapult_frame;
+        Spell humanturn_spell{ Spell::NONE };
+        bool humanturn_exit{ true };
+        bool humanturn_redraw{ true };
+        uint32_t animation_flags_frame{ 0 };
+        int catapult_frame{ 0 };
 
-        int _interruptAutoBattleForColor;
+        int _interruptAutoBattleForColor{ 0 };
 
         // The Channel ID of pre-battle sound. Used to check it is over to start the battle music.
         std::optional<int> _preBattleSoundChannelId{ -1 };
 
-        uint8_t _contourColor;
-        bool _brightLandType; // used to determine current monster contour cycling colors
-        uint32_t _contourCycle;
+        uint8_t _contourColor{ 110 };
 
-        const Unit * _currentUnit;
-        const Unit * _movingUnit;
-        const Unit * _flyingUnit;
-        const fheroes2::Sprite * b_current_sprite;
+        // True if background is bright. It is done to determine current unit contour cycling colors.
+        bool _brightLandType{ false };
+        uint32_t _contourCycle{ 0 };
+
+        const Unit * _currentUnit{ nullptr };
+        const Unit * _movingUnit{ nullptr };
+        const Unit * _flyingUnit{ nullptr };
+        const fheroes2::Sprite * b_current_sprite{ nullptr };
         fheroes2::Point _movingPos;
         fheroes2::Point _flyingPos;
 
-        int32_t index_pos;
+        int32_t _curentCellIndex{ -1 };
         // Index of the cell selected as the source for the Teleport spell
-        int32_t _teleportSpellSrcIdx;
-        fheroes2::Rect main_tower;
+        int32_t _teleportSpellSrcIdx{ -1 };
+        fheroes2::Rect _ballistaTowerRect;
 
         std::unique_ptr<StatusListBox> listlog;
 
@@ -475,7 +506,7 @@ namespace Battle
             uint32_t currentFrameId;
         };
 
-        BridgeMovementAnimation _bridgeAnimation;
+        BridgeMovementAnimation _bridgeAnimation{ false, BridgeMovementAnimation::UP_POSITION };
 
         struct SwipeAttack
         {
