@@ -29,7 +29,6 @@
 #include <functional>
 #include <list>
 #include <map>
-#include <memory>
 #include <ostream>
 #include <set>
 #include <string>
@@ -63,7 +62,7 @@
 #include "resource.h"
 #include "serialize.h"
 #include "settings.h"
-#include "world.h"
+#include "world.h" // IWYU pragma: associated
 #include "world_object_uid.h"
 
 namespace
@@ -210,7 +209,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
 
     // It is a valid case that a map has no add-ons.
     const size_t addonCount = fs.getLE32();
-    std::vector<MP2::mp2addon_t> vec_mp2addons( addonCount );
+    std::vector<MP2::MP2AddonInfo> vec_mp2addons( addonCount );
 
     if ( totalFileSize < MP2::MP2_MAP_INFO_SIZE + static_cast<size_t>( worldSize ) * MP2::MP2_TILE_STRUCTURE_SIZE + addonCount * MP2::MP2_ADDON_STRUCTURE_SIZE
                              + MP2::MP2_ADDON_COUNT_SIZE ) {
@@ -218,7 +217,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
         return false;
     }
 
-    for ( MP2::mp2addon_t & mp2addon : vec_mp2addons ) {
+    for ( auto & mp2addon : vec_mp2addons ) {
         MP2::loadAddon( fs, mp2addon );
     }
 
@@ -239,9 +238,9 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
     vec_object.reserve( 128 );
 
     for ( int32_t i = 0; i < worldSize; ++i ) {
-        Maps::Tiles & tile = vec_tiles[i];
+        Maps::Tile & tile = vec_tiles[i];
 
-        MP2::mp2tile_t mp2tile;
+        MP2::MP2TileInfo mp2tile;
         MP2::loadTile( fs, mp2tile );
         // There are some tiles which have object type as 65 and 193 which are Thatched Hut. This is exactly the same object as Peasant Hut.
         // Since the original number of object types is limited and in order not to confuse players we will convert this type into Peasant Hut.
@@ -276,14 +275,14 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
                 DEBUG_LOG( DBG_GAME, DBG_WARN, "Invalid MP2 format: incorrect addon index " << addonIndex )
                 break;
             }
-            tile.pushBottomLayerAddon( vec_mp2addons[addonIndex] );
-            tile.pushTopLayerAddon( vec_mp2addons[addonIndex] );
+            tile.pushGroundObjectPart( vec_mp2addons[addonIndex] );
+            tile.pushTopObjectPart( vec_mp2addons[addonIndex] );
             addonIndex = vec_mp2addons[addonIndex].nextAddonIndex;
         }
 
-        tile.AddonsSort();
+        tile.sortObjectParts();
 
-        if ( MP2::doesObjectNeedExtendedMetadata( tile.GetObject() ) ) {
+        if ( MP2::doesObjectNeedExtendedMetadata( tile.getMainObjectType() ) ) {
             vec_object.push_back( i );
         }
     }
@@ -314,31 +313,31 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
         switch ( castleType ) {
         case 0x00:
         case 0x80:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::KNGT ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::KNGT ) );
             break;
         case 0x01:
         case 0x81:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::BARB ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::BARB ) );
             break;
         case 0x02:
         case 0x82:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::SORC ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::SORC ) );
             break;
         case 0x03:
         case 0x83:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::WRLK ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::WRLK ) );
             break;
         case 0x04:
         case 0x84:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::WZRD ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::WZRD ) );
             break;
         case 0x05:
         case 0x85:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::NECR ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::NECR ) );
             break;
         case 0x06:
         case 0x86:
-            vec_castles.AddCastle( new Castle( posX, posY, Race::NONE ) );
+            vec_castles.AddCastle( std::make_unique<Castle>( posX, posY, Race::NONE ) );
             break;
         default:
             DEBUG_LOG( DBG_GAME, DBG_WARN,
@@ -446,8 +445,8 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
         const std::vector<uint8_t> pblock = fs.getRaw( blockSize );
 
         for ( const int32_t tileId : vec_object ) {
-            const Maps::Tiles & tile = vec_tiles[tileId];
-            if ( ( tile.getLayerType() & 0x3 ) != Maps::OBJECT_LAYER ) {
+            const Maps::Tile & tile = vec_tiles[tileId];
+            if ( ( tile.getMainObjectPart().layerType & 0x3 ) != Maps::OBJECT_LAYER ) {
                 continue;
             }
 
@@ -458,8 +457,8 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
         }
 
         if ( 0 <= objectTileId ) {
-            const Maps::Tiles & tile = vec_tiles[objectTileId];
-            const MP2::MapObjectType objectType = tile.GetObject();
+            const Maps::Tile & tile = vec_tiles[objectTileId];
+            const MP2::MapObjectType objectType = tile.getMainObjectType();
 
             switch ( objectType ) {
             case MP2::OBJ_CASTLE:
@@ -562,7 +561,7 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
                                    << "incorrect size block: " << pblock.size() )
                 }
                 else {
-                    std::pair<int, int> colorRace = Maps::getColorRaceFromHeroSprite( tile.GetObjectSpriteIndex() );
+                    std::pair<int, int> colorRace = Maps::getColorRaceFromHeroSprite( tile.getMainObjectPart().icnIndex );
                     const Kingdom & kingdom = GetKingdom( colorRace.first );
 
                     if ( colorRace.second == Race::RAND && colorRace.first != Color::NONE ) {
@@ -599,26 +598,28 @@ bool World::LoadMapMP2( const std::string & filename, const bool isOriginalMp2Fi
             case MP2::OBJ_SIGN:
             case MP2::OBJ_BOTTLE:
                 if ( MP2::MP2_SIGN_STRUCTURE_MIN_SIZE <= pblock.size() && 0x01 == pblock[0] ) {
-                    MapSign * obj = new MapSign();
+                    auto obj = std::make_unique<MapSign>();
                     obj->LoadFromMP2( objectTileId, pblock );
-                    map_objects.add( obj );
+
+                    map_objects.add( std::move( obj ) );
                 }
                 break;
             case MP2::OBJ_EVENT:
                 if ( MP2::MP2_EVENT_STRUCTURE_MIN_SIZE <= pblock.size() && 0x01 == pblock[0] ) {
-                    MapEvent * obj = new MapEvent();
+                    auto obj = std::make_unique<MapEvent>();
                     obj->LoadFromMP2( objectTileId, pblock );
-                    map_objects.add( obj );
+
+                    map_objects.add( std::move( obj ) );
                 }
                 break;
             case MP2::OBJ_SPHINX:
                 if ( MP2::MP2_RIDDLE_STRUCTURE_MIN_SIZE <= pblock.size() && 0x00 == pblock[0] ) {
-                    MapSphinx * obj = new MapSphinx();
+                    auto obj = std::make_unique<MapSphinx>();
                     obj->LoadFromMP2( objectTileId, pblock );
 
                     obj->validate();
 
-                    map_objects.add( obj );
+                    map_objects.add( std::move( obj ) );
                 }
                 break;
             default:
@@ -758,15 +759,23 @@ bool World::loadResurrectionMap( const std::string & filename )
                 assert( ( std::find( castleInfo.builtBuildings.begin(), castleInfo.builtBuildings.end(), BUILD_TENT ) != castleInfo.builtBuildings.end() )
                         == ( townObjects[object.index].metadata[1] == 0 ) );
 
-                Castle * castle = new Castle( static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width, race );
-                castle->SetColor( color );
-                castle->loadFromResurrectionMap( castleInfo );
+                fheroes2::Point castleCenter;
+                bool isCastle;
 
-                vec_castles.AddCastle( castle );
+                {
+                    auto castle = std::make_unique<Castle>( static_cast<int32_t>( tileId ) % width, static_cast<int32_t>( tileId ) / width, race );
+                    castle->SetColor( color );
+                    castle->loadFromResurrectionMap( castleInfo );
+
+                    castleCenter = castle->GetCenter();
+                    isCastle = castle->isCastle();
+
+                    vec_castles.AddCastle( std::move( castle ) );
+                }
 
                 if ( isRandom ) {
-                    Maps::UpdateCastleSprite( castle->GetCenter(), race, castle->isCastle(), true );
-                    Maps::ReplaceRandomCastleObjectId( castle->GetCenter() );
+                    Maps::UpdateCastleSprite( castleCenter, race, isCastle, true );
+                    Maps::ReplaceRandomCastleObjectId( castleCenter );
                 }
 
                 map_captureobj.Set( static_cast<int32_t>( tileId ), MP2::OBJ_CASTLE, color );
@@ -853,7 +862,7 @@ bool World::loadResurrectionMap( const std::string & filename )
                     }
 
                     // TODO: change MapEvent to support map format functionality.
-                    MapEvent * eventObject = new MapEvent();
+                    auto eventObject = std::make_unique<MapEvent>();
                     eventObject->resources = eventInfo.resources;
                     eventObject->artifact = eventInfo.artifact;
                     if ( eventInfo.artifact == Artifact::SPELL_SCROLL ) {
@@ -866,7 +875,8 @@ bool World::loadResurrectionMap( const std::string & filename )
                     eventObject->isSingleTimeEvent = !eventInfo.isRecurringEvent;
 
                     eventObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
-                    map_objects.add( eventObject );
+
+                    map_objects.add( std::move( eventObject ) );
 
                     break;
                 }
@@ -905,14 +915,14 @@ bool World::loadResurrectionMap( const std::string & filename )
                     assert( map.signMetadata.find( object.id ) != map.signMetadata.end() );
                     auto & signInfo = map.signMetadata[object.id];
 
-                    MapSign * signObject = new MapSign();
+                    auto signObject = std::make_unique<MapSign>();
                     signObject->message = std::move( signInfo.message );
                     signObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
                     if ( signObject->message.empty() ) {
                         signObject->setDefaultMessage();
                     }
 
-                    map_objects.add( signObject );
+                    map_objects.add( std::move( signObject ) );
 
                     break;
                 }
@@ -924,8 +934,7 @@ bool World::loadResurrectionMap( const std::string & filename )
                     assert( map.sphinxMetadata.find( object.id ) != map.sphinxMetadata.end() );
                     auto & sphinxInfo = map.sphinxMetadata[object.id];
 
-                    MapSphinx * sphinxObject = new MapSphinx();
-
+                    auto sphinxObject = std::make_unique<MapSphinx>();
                     sphinxObject->riddle = std::move( sphinxInfo.riddle );
 
                     for ( auto & answer : sphinxInfo.answers ) {
@@ -952,7 +961,7 @@ bool World::loadResurrectionMap( const std::string & filename )
                     // However, it seems logically incorrect.
                     sphinxObject->isTruncatedAnswer = false;
 
-                    map_objects.add( sphinxObject );
+                    map_objects.add( std::move( sphinxObject ) );
 
                     break;
                 }
@@ -971,14 +980,14 @@ bool World::loadResurrectionMap( const std::string & filename )
                     assert( map.signMetadata.find( object.id ) != map.signMetadata.end() );
                     auto & signInfo = map.signMetadata[object.id];
 
-                    MapSign * signObject = new MapSign();
+                    auto signObject = std::make_unique<MapSign>();
                     signObject->message = std::move( signInfo.message );
                     signObject->setUIDAndIndex( static_cast<int32_t>( tileId ) );
                     if ( signObject->message.empty() ) {
                         signObject->setDefaultMessage();
                     }
 
-                    map_objects.add( signObject );
+                    map_objects.add( std::move( signObject ) );
                 }
             }
             else if ( object.group == Maps::ObjectGroup::ADVENTURE_ARTIFACTS ) {
@@ -1124,10 +1133,10 @@ bool World::loadResurrectionMap( const std::string & filename )
 
 bool World::ProcessNewMP2Map( const std::string & filename, const bool checkPoLObjects )
 {
-    for ( Maps::Tiles & tile : vec_tiles ) {
-        Maps::Tiles::fixMP2MapTileObjectType( tile );
+    for ( Maps::Tile & tile : vec_tiles ) {
+        Maps::Tile::fixMP2MapTileObjectType( tile );
 
-        if ( !updateTileMetadata( tile, tile.GetObject(), checkPoLObjects ) ) {
+        if ( !updateTileMetadata( tile, tile.getMainObjectType(), checkPoLObjects ) ) {
             ERROR_LOG( "Failed to load The Price of Loyalty map '" << filename << "' which is not supported by this version of the game." )
             // You are trying to load a PoL map named as a MP2 file.
             return false;
@@ -1144,7 +1153,7 @@ bool World::ProcessNewMP2Map( const std::string & filename, const bool checkPoLO
 
     // Search for a tile with a predefined Ultimate Artifact
     const auto ultArtTileIter
-        = std::find_if( vec_tiles.begin(), vec_tiles.end(), []( const Maps::Tiles & tile ) { return tile.isSameMainObject( MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT ); } );
+        = std::find_if( vec_tiles.begin(), vec_tiles.end(), []( const Maps::Tile & tile ) { return tile.getMainObjectType() == MP2::OBJ_RANDOM_ULTIMATE_ARTIFACT; } );
     int32_t ultimateArtifactTileId = -1;
     int32_t ultimateArtifactRadius = 0;
     if ( ultArtTileIter != vec_tiles.end() ) {
@@ -1152,13 +1161,12 @@ bool World::ProcessNewMP2Map( const std::string & filename, const bool checkPoLO
         ultimateArtifactRadius = static_cast<int32_t>( ultArtTileIter->metadata()[0] );
 
         // Remove the predefined Ultimate Artifact object
-        ultArtTileIter->Remove( ultArtTileIter->GetObjectUID() );
-        ultArtTileIter->setAsEmpty();
+        ultArtTileIter->removeObjectPartsByUID( ultArtTileIter->getMainObjectPart()._uid );
     }
 
     setUltimateArtifact( ultimateArtifactTileId, ultimateArtifactRadius );
 
-    PostLoad( true );
+    PostLoad( true, false );
 
     vec_kingdoms.ApplyPlayWithStartingHero();
 
@@ -1167,7 +1175,7 @@ bool World::ProcessNewMP2Map( const std::string & filename, const bool checkPoLO
     return true;
 }
 
-bool World::updateTileMetadata( Maps::Tiles & tile, const MP2::MapObjectType objectType, const bool checkPoLObjects )
+bool World::updateTileMetadata( Maps::Tile & tile, const MP2::MapObjectType objectType, const bool checkPoLObjects )
 {
     switch ( objectType ) {
     case MP2::OBJ_ARTIFACT:
@@ -1249,7 +1257,7 @@ bool World::updateTileMetadata( Maps::Tiles & tile, const MP2::MapObjectType obj
     case MP2::OBJ_FIRE_ALTAR:
     case MP2::OBJ_WATER_ALTAR:
         // We need to clear metadata because it is being stored as a part of an MP2 object.
-        resetObjectInfoOnTile( tile );
+        resetObjectMetadata( tile );
         updateObjectInfoTile( tile, true );
         break;
 
@@ -1259,8 +1267,8 @@ bool World::updateTileMetadata( Maps::Tiles & tile, const MP2::MapObjectType obj
 
     case MP2::OBJ_HERO: {
         // remove map editor sprite
-        if ( tile.getObjectIcnType() == MP2::OBJ_ICN_TYPE_MINIHERO ) {
-            tile.Remove( tile.GetObjectUID() );
+        if ( tile.getMainObjectPart().icnType == MP2::OBJ_ICN_TYPE_MINIHERO ) {
+            tile.removeObjectPartsByUID( tile.getMainObjectPart()._uid );
         }
 
         Heroes * chosenHero = GetHeroes( Maps::GetPoint( tile.GetIndex() ) );
@@ -1284,7 +1292,7 @@ bool World::updateTileMetadata( Maps::Tiles & tile, const MP2::MapObjectType obj
             }
         }
 
-        const MP2::MapObjectType updatedObjectType = tile.GetObject( false );
+        const MP2::MapObjectType updatedObjectType = tile.getMainObjectType( false );
         if ( updatedObjectType != objectType ) {
             return updateTileMetadata( tile, updatedObjectType, checkPoLObjects );
         }
@@ -1316,7 +1324,7 @@ void World::setUltimateArtifact( const int32_t tileId, const int32_t radius )
             return false;
         }
 
-        return GetTiles( idx ).GoodForUltimateArtifact();
+        return getTile( idx ).GoodForUltimateArtifact();
     };
 
     if ( tileId < 0 ) {
@@ -1324,7 +1332,7 @@ void World::setUltimateArtifact( const int32_t tileId, const int32_t radius )
         std::vector<int32_t> pool;
         pool.reserve( vec_tiles.size() / 2 );
 
-        for ( const Maps::Tiles & tile : vec_tiles ) {
+        for ( const Maps::Tile & tile : vec_tiles ) {
             const int32_t idx = tile.GetIndex();
 
             if ( checkTileForSuitabilityForUltArt( idx ) ) {
@@ -1392,7 +1400,7 @@ void World::addDebugHero()
     const fheroes2::Point & cp = castle->GetCenter();
     Heroes * hero = vec_heroes.Get( Heroes::DEBUG_HERO );
 
-    if ( hero && !GetTiles( cp.x, cp.y + 1 ).getHero() ) {
+    if ( hero && !getTile( cp.x, cp.y + 1 ).getHero() ) {
         hero->Recruit( castle->GetColor(), { cp.x, cp.y + 1 } );
     }
 }
